package de.uulm.omi.dbms.workload.api;

import de.uulm.omi.cloudiator.dbms.evaluator.model.*;
import de.uulm.omi.dbms.workload.api.WorkloadApiService;
import de.uulm.omi.dbms.workload.api.factories.WorkloadApiServiceFactory;

import io.swagger.annotations.ApiParam;
import io.swagger.jaxrs.*;

import de.uulm.omi.cloudiator.dbms.evaluator.model.Error;
import java.io.File;
import de.uulm.omi.cloudiator.dbms.evaluator.model.TPCCWorkload;
import de.uulm.omi.cloudiator.dbms.evaluator.model.TSBSWorkload;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadProcessInstance;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadProcessSubmissionError;
import de.uulm.omi.cloudiator.dbms.evaluator.model.YCSBWorkload;

import java.util.Map;
import java.util.List;
import de.uulm.omi.dbms.workload.api.NotFoundException;

import java.io.InputStream;

import org.glassfish.jersey.media.multipart.FormDataContentDisposition;
import org.glassfish.jersey.media.multipart.FormDataParam;

import javax.servlet.ServletConfig;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.*;
import javax.validation.constraints.*;

@Path("/workload")

@Produces({ "application/json" })
@io.swagger.annotations.Api(description = "the workload API")
@javax.annotation.Generated(value = "io.swagger.codegen.languages.JavaJerseyServerCodegen", date = "2020-05-19T16:43:06.028+02:00")
public class WorkloadApi  {
   private final WorkloadApiService delegate;

   public WorkloadApi(@Context ServletConfig servletContext) {
      WorkloadApiService delegate = null;

      if (servletContext != null) {
         String implClass = servletContext.getInitParameter("WorkloadApi.implementation");
         if (implClass != null && !"".equals(implClass.trim())) {
            try {
               delegate = (WorkloadApiService) Class.forName(implClass).newInstance();
            } catch (Exception e) {
               throw new RuntimeException(e);
            }
         } 
      }

      if (delegate == null) {
         delegate = WorkloadApiServiceFactory.getWorkloadApi();
      }

      this.delegate = delegate;
   }

    @GET
    @Path("/result")
    
    @Produces({ "text/plain" })
    @io.swagger.annotations.ApiOperation(value = "get the result file of a workload task", notes = "Start a YCSB for the specified database ", response = File.class, tags={  })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "Returns a result text file", response = File.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response workloadResultGet(@ApiParam(value = "the id of the worklaod execution task, used as prefix for result files",required=true) @QueryParam("taskId") String taskId
,@ApiParam(value = "workload type of the result file",required=true, allowableValues="YCSB, TPCC, TSBS") @QueryParam("workloadType") String workloadType
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.workloadResultGet(taskId,workloadType,securityContext);
    }
    @GET
    @Path("/status")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "get the current state of the workloadProcess", notes = "Start a YCSB for the specified database ", response = WorkloadProcessInstance.class, tags={  })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the last workloadProcess", response = WorkloadProcessInstance.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response workloadStatusGet(@ApiParam(value = "the applicationInstanceI of the deployed DB cluster via Cloudiator") @QueryParam("applicationInstanceId") String applicationInstanceId
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.workloadStatusGet(applicationInstanceId,securityContext);
    }
    @POST
    @Path("/tpcc")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "starts a TPC-C workload against a relational DBMS", notes = "Start a TPC-C worklaod for the specified database based on the TPC-C implementation of cockroachdb ", response = WorkloadProcessInstance.class, tags={  })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = WorkloadProcessInstance.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already a workloadProcess running", response = WorkloadProcessSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response workloadTpccPost(@ApiParam(value = "the id of the worklaod execution task, used as prefix for result files",required=true) @QueryParam("taskId") String taskId
,@ApiParam(value = "the TPC-C workload configuration" ,required=true) TPCCWorkload workload
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.workloadTpccPost(taskId,workload,securityContext);
    }
    @POST
    @Path("/tsbs/executeQueries")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "executes TSBS queries based on the loaded data", notes = "Execute queries ", response = WorkloadProcessInstance.class, tags={  })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = WorkloadProcessInstance.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already a workloadProcess running", response = WorkloadProcessSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response workloadTsbsExecuteQueriesPost(@ApiParam(value = "the id of the workload execution task, used as prefix for result files",required=true) @QueryParam("taskId") String taskId
,@ApiParam(value = "the TSBS workload configuration" ,required=true) TSBSWorkload workload
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.workloadTsbsExecuteQueriesPost(taskId,workload,securityContext);
    }
    @POST
    @Path("/tsbs/generateData")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "generates the TSBS data", notes = "Generate data ", response = WorkloadProcessInstance.class, tags={  })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = WorkloadProcessInstance.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already a workloadProcess running", response = WorkloadProcessSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response workloadTsbsGenerateDataPost(@ApiParam(value = "the id of the workload execution task, used as prefix for result files",required=true) @QueryParam("taskId") String taskId
,@ApiParam(value = "the TSBS workload configuration" ,required=true) TSBSWorkload workload
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.workloadTsbsGenerateDataPost(taskId,workload,securityContext);
    }
    @POST
    @Path("/tsbs/generateQueries")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "generates the TSBS quires in the DBMS-specific format", notes = "Generate TSBS queries ", response = WorkloadProcessInstance.class, tags={  })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = WorkloadProcessInstance.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already a workloadProcess running", response = WorkloadProcessSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response workloadTsbsGenerateQueriesPost(@ApiParam(value = "the id of the workload execution task, used as prefix for result files",required=true) @QueryParam("taskId") String taskId
,@ApiParam(value = "the TSBS workload configuration" ,required=true) TSBSWorkload workload
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.workloadTsbsGenerateQueriesPost(taskId,workload,securityContext);
    }
    @POST
    @Path("/tsbs/loadData")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "loads the data in the specified timeseries DBMS", notes = "Load data ", response = WorkloadProcessInstance.class, tags={  })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = WorkloadProcessInstance.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already a workloadProcess running", response = WorkloadProcessSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response workloadTsbsLoadDataPost(@ApiParam(value = "the id of the workload execution task, used as prefix for result files",required=true) @QueryParam("taskId") String taskId
,@ApiParam(value = "the TSBS workload configuration" ,required=true) TSBSWorkload workload
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.workloadTsbsLoadDataPost(taskId,workload,securityContext);
    }
    @POST
    @Path("/ycsb")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "start a YCSB workload against the provided database", notes = "Start a YCSB for the specified database ", response = WorkloadProcessInstance.class, tags={  })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the state of the started workloadProcess", response = WorkloadProcessInstance.class),
        
        @io.swagger.annotations.ApiResponse(code = 403, message = "already a workloadProcess running", response = WorkloadProcessSubmissionError.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response workloadYcsbPost(@ApiParam(value = "the id of the worklaod execution task, used as prefix for result files",required=true) @QueryParam("taskId") String taskId
,@ApiParam(value = "the YCSB workload configuration" ,required=true) YCSBWorkload workload
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.workloadYcsbPost(taskId,workload,securityContext);
    }
}
