package de.uulm.omi.workload;

import static com.google.common.base.Preconditions.checkNotNull;

import java.io.File;
import java.io.IOException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Daniel Seybold on 16.01.2018.
 */
public enum WorkloadProcess implements WorkloadProcessService {


  INSTANCE{

    Process process;
    String processId;
    long timestamp;

    @Override
    public Process startWorkload(String workloadBinariesDirectory, String shell, String binary, String args, String processId, long timestamp) {

      checkNotNull(workloadBinariesDirectory);
      checkNotNull(shell);
      checkNotNull(binary);
      checkNotNull(args);

      this.timestamp = timestamp;
      this.processId = processId;

      ProcessBuilder builder = new ProcessBuilder();

      try {
        File directory = new File(workloadBinariesDirectory);
        this.process = builder
            .directory(directory)
            .command(shell, binary, args).start();
        LOGGER.debug("Process started: " + shell + " " + binary + " " + args);

       //writeOutput();

        return process;

      } catch (IOException e) {
        LOGGER.error("Error while starting process: " + shell + " " + binary + " " + args, e);

        return null;
      }
    }




    @Override
    public boolean isRunning() {
      if(process != null && process.isAlive()) return true;
      else return false;

    }

    @Override
    public int getExitCode() {
      if(process == null){
        LOGGER.warn("No process started yet, returning ExitCode 0");
        return 0;
      }
      return process.exitValue();
    }

    @Override
    public boolean killProcess() {
      LOGGER.debug("destroying process...");
      process.destroy();
      LOGGER.debug("process destroyed!");
      return true;
    }

    @Override
    public String getProcessId() {
      return this.processId;
    }

    @Override
    public long getStartTimestamp() {
      return this.timestamp;
    }


  };

  private  static final Logger LOGGER = LoggerFactory.getLogger(WorkloadProcess.class.getName());

  public static WorkloadProcess getInstance() {
    return WorkloadProcess.INSTANCE;
  }

}
